#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Date: 2022/8/17
# @Author: pcl

class PanguEvolutionDTO(object):

    DEFAULT_MAX_TOKEN = 50
    TOP_P = 0.0
    TOP_K = 1

    def __init__(self):
        pass

    @classmethod
    def build_request(cls, prompt_input, max_token, top_p, top_k):
        max_token = cls.DEFAULT_MAX_TOKEN if max_token is None else max_token
        top_p = cls.TOP_P if top_p is None else top_p
        top_k = cls.TOP_K if top_k is None else top_k

        request = {"u": prompt_input,
                   "result_len": max_token,
                   "top_p": top_p,
                   "top_k": top_k,
                   "task_name": ""
                   }
        return request

    @classmethod
    def build_default_response(cls, api_key, model, prompt_input):
        default_response = {
            "id": api_key,
            "model": model,
            "object": "generate",
            "results": {
                "prompt_input": prompt_input,
                "generate_text": None,
                "logprobs": None,
            },
            "status": False
        }
        return default_response


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Date: 2022/8/17
# @Author: pcl
import requests
import time

default_response = {
    "id": None,
    "model": None,
    "object": "generate",
    "results": {
        "prompt_input": None,
        "generate_text": None,
        "logprobs": None,
    },
    "status": True
}

def reset_default_response():
    global default_response
    default_response = {
        "id": None,
        "model": None,
        "object": "generate",
        "results": {
            "prompt_input": None,
            "generate_text": None,
            "logprobs": None,
        },
        "status": True
    }

def send_requests_pangu_alpha(payload):
    global default_response
    response = requests.get('https://pangu-alpha.openi.org.cn/query?', params=payload)
    if response.status_code == 200:
        result = response.json()['rsvp']
        if result is None:
            payload['isWaiting'] = 'true'
            time.sleep(10)
            send_requests_pangu_alpha(payload)
        else:
            default_response['results']['prompt_input'] = payload['u']
            default_response['results']['generate_text'] = result[-1]
    else:
        default_response['status'] = False
        print("Error response! checkout your [url] is 'https://pangu-alpha.openi.org.cn/query?'\n")

def get_response():
    return default_response


# coding=utf-8
# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .distributed import *
from .bert_model import BertModel
from .realm_model import ICTBertModel
from .gpt2_model import GPT2Model
from .alpha_enhanced_model import EnhancedModel
from .utils import get_params_for_weight_decay_optimization
from .language_model import get_language_model



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Date: 2022/8/16
# @Author: pcl
import requests
from pcl_pangu.online.infer.pangu_alpha_dto import reset_default_response, send_requests_pangu_alpha, get_response
from pcl_pangu.online.infer.pangu_evolution_dto import PanguEvolutionDTO

class Infer(object):

    pangu_evolution_url = "https://pangu-alpha.openi.org.cn/query_advanced?"

    def __init__(self):
        pass

    @classmethod
    def do_generate_pangu_alpha(cls, model, prompt_input, max_token=None, top_k=None, top_p=None, api_key=None, **kwargs):
        payload = {
            'u': prompt_input,
            'top_k': top_k,
            'top_p': top_p,
            'result_len': max_token,
            'isWaiting': 'false'
        }
        reset_default_response()

        send_requests_pangu_alpha(payload)
        result_response = get_response()
        result_response['id'] = api_key
        result_response['model'] = model

        return result_response

    @classmethod
    def do_generate_pangu_evolution(cls, model, prompt_input, max_token=None, top_k=None, top_p=None, api_key=None, **kwargs):

        request = PanguEvolutionDTO.build_request(prompt_input, max_token, top_p, top_k)
        default_response = PanguEvolutionDTO.build_default_response(api_key, model, prompt_input)

        response = requests.get(cls.pangu_evolution_url, params=request)
        if response.status_code == 200:
            result = response.json()["rsvp"]
            if result:
                default_response["results"]["generate_text"] = result[-1]
                default_response["status"] = True
                return default_response

        print("Error response!")
        return default_response

    @classmethod
    def generate(cls, model, prompt_input, max_token=None, top_k=None, top_p=None, api_key=None, **kwargs):
        """
        model: 模型
        prompt_input: 文本输入，可以结合prompt做为整体输入
        max_token:
        top_k: 随机采样参数
        top_p: 随机采样参数
        kwargs: 不同模型支持的其他参数
        """
        if "pangu-alpha-13B-md"==model:
            return cls.do_generate_pangu_alpha(model, prompt_input, max_token, top_k, top_p, api_key, **kwargs)

        elif "pangu-alpha-evolution-2B6-pt"==model:
            return cls.do_generate_pangu_evolution(model, prompt_input, max_token, top_k, top_p, api_key, **kwargs)

        else:
            defalut_response = {"status": "The model does not exist."}
            print("Error model.")
            return defalut_response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Date: 2022/7/20
# @Author: 2022 PCL
import os
import sys
from loguru import logger

from pcl_pangu.context import check_context
from pcl_pangu.model.launcher_torch import launch
from pcl_pangu.model.evolution.config_evolution import DISTRUBUTED_CONFIG, model_config_gpu


def train(config):
    print('----------------------------- training config -----------------------------')
    print("> Base Model: [evolution]")
    print("> Model Size: [{}]".format(config.model))
    print("> data_path: {}".format(config.data_path))
    print("> global batch_size: {}".format(config.batch_size))
    print("> save to path: {}".format(config.save))
    print('------------------------------ end of config -----------------------------')

    if check_context()=='pytorch':
        script_args = config._get_training_script_args()
        py_script = '/panguAlpha_pytorch/pretrain_evolution.py'
        run_pt(script_args, py_script)

    else:
        print("ERROR: wrong backend.")
        return 1

def fine_tune(config):
    print('--------------------------- finetune config -----------------------------')
    print("> Base Model: [evolution]")
    print("> Model Size: [{}]".format(config.model))
    print("> data_path: {}".format(config.data_path))
    print("> global batch_size: {}".format(config.batch_size))
    print("> save to path: {}".format(config.save))
    print('---------------------------- end of config -------------------------------')

    if check_context()=='pytorch':
        script_args = config._get_training_script_args()
        py_script = '/panguAlpha_pytorch/pretrain_evolution.py'
        run_pt(script_args, py_script)

    else:
        print("ERROR: wrong backend.")
        return 1

def inference(config,top_k=1,top_p=0.9,input=None,input_file=None,output_file=None,
              generate_max_tokens=128,oneCardInference=True):
    assert generate_max_tokens > 0 and generate_max_tokens<=800, "> generate_max_tokens always in (0, 800]"
    print('--------------------------- inference config --------------------------')
    print("> Base Model: [evolution]")
    print("> Model Size: [{}]".format(config.model))
    print("> global batch_size: {}".format(config.batch_size))
    print("> generate_max_tokens length: {}".format(generate_max_tokens))
    print('---------------------------- end of config ----------------------------')

    if check_context()=='pytorch':
        from .config_evolution import DEFAULT_CONFIG
        DEFAULT_CONFIG['finetune'] = True
        config.batch_size = 1
        script_args = config._get_training_script_args(oneCardInference=oneCardInference)
        py_script = '/panguAlpha_pytorch/tools/generate_samples_epangu.py'
        script_args.append('--top-k={}'.format(top_k))
        script_args.append('--top-p={}'.format(top_p))
        if input is not None:
            script_args.append('--sample-input={}'.format(input.encode('utf-8').hex()))
        if input_file is not None:
            script_args.append('--sample-input-file={}'.format(input_file))
        if output_file is not None:
            script_args.append('--sample-output-file={}'.format(output_file))
        if generate_max_tokens is not None:
            script_args.append('--generate_max_tokens={}'.format(generate_max_tokens))

        script_args.append('--isEvolution=True')
        run_pt(script_args, py_script)

    else:
        print("ERROR: wrong backend.")
        return 1


def run_pt(script_args, py_script, **kwargs):
    current_dir = os.path.dirname(os.path.dirname(__file__))
    sys.path.append(current_dir + '/panguAlpha_pytorch')

    py_script = current_dir + py_script
    logger.info("> Running {} with args: {}".format(py_script, script_args))

    launch(training_script=py_script,
           training_script_args=script_args,
           **DISTRUBUTED_CONFIG,
           **kwargs)
    return 0


if __name__ == '__main__':
    config = model_config_gpu()
    inference(config)
